// Source Code for DeleteUserDialog.class //
// Client Code for Desktop Calendar       //

// Code based on using Server #18     //
// Written by Khalid Mufti - Group 31 //

import java.io.*;
import java.awt.*;
import java.awt.event.*;

// This class handles the dialog which enables the admin to delete a    //
// user from the server.						//
// Inherits from the Dialog class (modal) and requires the parent frame //
// and a socket connected to the server, as parameters                  //  
public class DeleteUserDialog extends Dialog implements ActionListener
{
   private GridBagLayout gbLayout;
   private GridBagConstraints gbConstraints; 
   private TextField uname;
   private Button cancel, ok, help;
   private Label uname_l;
   private Label dum1;
   private ClientSocket clientsocket;
   private Frame frame;
   private MsgBox msgbox;
   private AdminHelpBox helpbox;
   private StringBuffer str;


   // The constructor for the class.  Requires a parent frame and socket connected //
   // to the server as parameters.                                                 //
   public DeleteUserDialog(Frame parent, ClientSocket cs)
   {
     // Call Dialog class constructor //
     super (parent, "  Enter username to Delete", true);

     // Set window size and type and copy parameters to local variables //
     setSize(300,300);
     setResizable(false);  
     clientsocket = cs;
     frame=parent;

     // Instantiate grid for layout purposes //
     gbLayout = new GridBagLayout();
     gbConstraints = new GridBagConstraints();
     setLayout (gbLayout); 

     // Allocate memory for all new items in Dialog, and add action listeners //     
     dum1 = new Label ("");  //used to provide spacing//
     uname = new TextField (20);
     uname.addActionListener (this);
     cancel = new Button ("Cancel");
     cancel.addActionListener (this);
     ok = new Button ("OK");
     ok.addActionListener (this);
     help = new Button ("Help");
     help.addActionListener (this);
     uname_l = new Label("Login Name:");
  
     // Now place the components in the Dialog, based on the grid bag layout //
     gbConstraints.fill = GridBagConstraints.HORIZONTAL;
     addObj (uname,        0, 1, 2, 1);
     addObj (uname_l,      0, 0, 1, 1);
     addObj (dum1,         1, 0, 2, 1);        
     gbConstraints.fill = GridBagConstraints.BOTH;
     addObj (help,         2, 0, 1, 2);
     addObj (ok,           2, 1, 2, 1);
     addObj (cancel,       3, 1, 2, 1);        

     // Add a listener for closing the window prematurely and set the textfields //
     // to an empty value                                   		         //
     addWindowListener (new CloseWindow());
     uname.setText("");
   } 

   // This function handles any action events from the user //
   public void actionPerformed (ActionEvent e)
   {   
      // If user pressed the cancel button //
      if ((e.getSource())==cancel)
         dispose();

      // If user pressed the ok button //
      else if ((e.getSource())==ok)
      {
        try 
        {
           // Send request to server //
           clientsocket.Write("removeuser," + uname.getText());
           String buf = new String (clientsocket.Read()); 
       
           // Parse numerical response from server // 
           if (buf.regionMatches(false, 0 , "201", 0,  3))
           {
             // Remove the _personal login as well (that handled personal events) //
             clientsocket.Write("removeuser," + uname.getText() + "_personal");
             String garbage = new String (clientsocket.Read()); 
       
             msgbox = new MsgBox (frame, "  SUCCESS", "User Deleted Successfully!", 250, 150);
             msgbox.setVisible(true);
             dispose();
           }     

           else if (buf.regionMatches(false, 0 , "802", 0,  3))
           {
             msgbox = new MsgBox (frame, "  DELETE ERROR", "Admin cannot be Deleted!", 250, 150);
             msgbox.setVisible(true);
           }     

           else if (buf.regionMatches(false, 0 , "803", 0,  3))
           {
             msgbox = new MsgBox (frame, "  DELETE ERROR", "User is On-Line!", 250, 150);
             msgbox.setVisible(true);
           }     

           else 
           {
             msgbox = new MsgBox (frame, "  DELETE ERROR", "Invalid Entry Detected!", 250, 150);
             msgbox.setVisible(true);
           }     
         }
         
         catch (IOException io)  // Connection must have been lost //
         {
           msgbox = new MsgBox (frame, "  NETWORK ERROR", "Network Error Detected!", 250, 150);
           msgbox.setVisible(true);
           dispose();
         }  
      }    

      // If user pressed the help button //
      else if ((e.getSource())==help)
      {
         helper();
      }    
   }
  
   // This function simply adds an object (registers it) to the Dialog //
   // Further, it sets various parts of the grid bag layout            // 
   private void addObj (Component c, int row, int column, int width, int height)
   {
      gbConstraints.gridx=column;
      gbConstraints.gridy=row;   
      gbConstraints.gridwidth=width;
      gbConstraints.gridheight=height;
      gbLayout.setConstraints (c, gbConstraints);
      add (c);
   }

   // This function simply pops up a help dialog box that displays help on using //
   // this dialog screen.         
   private void helper()
   {
      // Set what the help box should say //
      str = new StringBuffer(); 
      str.append("This screen is used to delete an existing user\n");
      str.append("from the desktop server.\n");
      str.append("\n");
      str.append("For the login id field, enter the username\n");
      str.append("to be deleted.\n");
      str.append("\n");
      str.append("When done, cick on the OK button; otherwise,\n");
      str.append("click Cancel to return to the Admin menu.\n");
     
      // Create and show modal help box //
      helpbox = new AdminHelpBox (frame, "  Help Screen", str.toString(), true, 300, 300);
      helpbox.setVisible(true); 
   }
}