// Source Code for ViewRoomDialog.class //
// Client Code for Desktop Calendar    //

// Code based on using Server #18     //
// Written by Khalid Mufti - Group 31 //

import java.io.*;
import java.awt.*;
import java.util.*;
import java.awt.event.*;

// This class handles the dialog which enables a user to view a list    //
// of current rooms and their capacity on the server.                   //
// Inherits from the Dialog class (modal) and requires the parent frame //
// and a socket connected to the server, as parameters                  //  
public class ViewRoomDialog extends Dialog implements ActionListener
{
   private GridBagLayout gbLayout;
   private GridBagConstraints gbConstraints; 
   private Button ok, help;
   private List roomlist;
   private Label dum1;
   private ClientSocket clientsocket;
   private Frame frame;
   private MsgBox msgbox;
   private StringTokenizer tokens;
   private AdminHelpBox helpbox;
   private StringBuffer str;
   
   // The constructor for the class.  Requires a parent frame and socket connected //
   // to the server as parameters.                                                 //
   public ViewRoomDialog(Frame parent, ClientSocket cs)
   {
     // Call Dialog class constructor //
     super (parent, "  Current Room List", true);

     // Set window size and type and copy parameters to local variables //
     setSize(350,300);
     setResizable(false);  
     frame = parent;
     clientsocket = cs;

     // Instantiate grid for layout purposes //
     gbLayout = new GridBagLayout();
     gbConstraints = new GridBagConstraints();
     setLayout (gbLayout); 
     
     // Allocate memory for all new items in Dialog, and add action listeners //
     dum1 = new Label (""); //used to provide spacing//
     ok = new Button ("OK");
     ok.addActionListener (this);
     help = new Button ("Help");
     help.addActionListener (this);
 
     // Now place the components in the Dialog, based on the grid bag layout //
     gbConstraints.fill = GridBagConstraints.HORIZONTAL;
     addObj (dum1,        10, 0, 3, 1);        
     gbConstraints.fill = GridBagConstraints.BOTH;
     addObj (help,        11, 0, 1, 2);
     addObj (ok,          11, 1, 2, 1);
 
     // Add a listener for closing the window prematurely //
     addWindowListener (new CloseWindow());

     // Get list of rooms from server //
     try 
     {
        // Send request to server //
        clientsocket.Write("listallrooms,");
        String buf = new String (clientsocket.Read()); 

        // Parse numerical response from server //     
        if (buf.regionMatches(false, 0 , "192", 0,  3))
        {
           // Tokenize string to extract room names and capacity //
           tokens = new StringTokenizer (buf, ",");
           roomlist = new List (5, false);
           tokens.nextToken(); // Throw away response number //
           while (true)
           { 
              String rbuffer = new String (tokens.nextToken()); 
              if (rbuffer.regionMatches(false, 0 , "END", 0,  3)) // End of room list //
 	         break;	            

              String nbuffer = new String (tokens.nextToken());
              roomlist.add (rbuffer + " holds " + nbuffer + " people"); // Add to list //
           }

           addObj (roomlist, 0, 0, 2, 1);     
        }     

        else 
        {
           msgbox = new MsgBox (frame, "  UNKNOWN ERROR", "Unknown Error Detected!", 250, 150);
           msgbox.setVisible(true);
           dispose();
        }
     } 
      
     catch (IOException io)   // Connection must have been lost //
     {
        msgbox = new MsgBox (frame, "  NETWORK ERROR", "Network Error Detected!", 250, 150);
        msgbox.setVisible(true);
        dispose();
     }  
   } 

   // This function handles any action events from the user //
   public void actionPerformed (ActionEvent e)
   {   
      // If user pressed the ok button //
      if ((e.getSource())==ok)
      {
         dispose();      
      }    
    
      // If user pressed the help button //
      else if ((e.getSource())==help)
      {
         helper();
      }    
   }
 
   // This function simply adds an object (registers it) to the Dialog //
   // Further, it sets various parts of the grid bag layout            //  
   private void addObj (Component c, int row, int column, int width, int height)
   {
      gbConstraints.gridx=column;
      gbConstraints.gridy=row;   
      gbConstraints.gridwidth=width;
      gbConstraints.gridheight=height;
      gbLayout.setConstraints (c, gbConstraints);
      add (c);
   }
   
   // This function simply pops up a help dialog box that displays help on using //
   // this dialog screen.                                                        //
   private void helper()
   {
      // Set what the help box should say //
      str = new StringBuffer(); 
      str.append("This screen lists all the rooms on the desktop\n");
      str.append("server, as well as their capacity.  Although this\n");
      str.append("server does not allow modifying or deleting rooms,\n");
      str.append("this list is present simply to give an idea of the\n");
      str.append("rooms the server knows about.\n");
      str.append("\n");
      str.append("Once you are finished looking at the list,\n");
      str.append("click OK to return to the Admin menu.\n");

      // Create and show modal help box //
      helpbox = new AdminHelpBox (frame, "  Help Screen", str.toString(), true, 300, 300);
      helpbox.setVisible(true); 
   }
}